// DMXhid.cpp
//

/* Copyright Notice **********************************************************
 * 
 *  Copyright (c) 2004. EdgeView Software, LLC.
 *  All Rights Reserved.
 *  
 *  Permission to use, copy, modify, and distribute this software and its
 *  documentation for any purpose, without fee, and without a written
 *  agreement, is hereby granted, provided that the above copyright
 *  notice, this paragraph and the following three paragraphs appear in
 *  all copies, modifications, and distributions.
 * 
 *  Created by EdgeView Software, LLC. 
 *  For technical information, contact support@edgeviewsoftware.com.
 *  
 *  IN NO EVENT SHALL EDGEVIEW SOFTWARE BE LIABLE TO ANY PARTY FOR DIRECT, 
 *  INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST
 *  PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, 
 *  EVEN IF EDGEVIEW SOFTWARE HAS BEEN ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGES.
 * 
 *  THIS SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF ANY, IS PROVIDED "AS
 *  IS".  EDGEVIEW SOFTWARE HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 *  UPDATES, ENHANCEMENTS, OR MODIFICATIONS TO THIS SOFTWARE.  EDGEVIEW SOFTWARE
 *  SPECIFICALLY DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE. WITHOUT LIMITING THE GENERALITY OF
 *  THE FOREGOING, THIS SOFTWARE IS NOT DESIGNED, MANUFACTURED OR
 *  INTENDED FOR USE OR RESALE IN CONNECTION WITH APPLICATIONS REQUIRING
 *  FAIL SAFE PERFORMANCE (INCLUDING, BUT NOT LIMITED TO, USE IN
 *  HAZARDOUS ENVIRONMENTS, OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 *  NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 *  SUPPORT MACHINES, OR WEAPONS SYSTEMS).
 * 
 * End Copyright Notice ******************************************************/

#include <windows.h>
#include <stdio.h>
#include <malloc.h>

// the next three includes access the USB HID interface
#include <setupapi.h>                    
extern "C" {
#include </98DDK/src/hid/inc/hidsdi.h>    /* note: part of Windows DDK */
#include </98DDK/src/hid/inc/hidpi.h>    /* note: also part of Windows DDK */
}

#include "DMXhid.h"

/* global constants */

// vendor ID (Durand Interstellar, Inc.)
#define VID 0x0F48
// product ID
#define PID 0x4201

bool DMX_adapter_found = false;

static int outputReportLength;
static char * outbuffer;
static char * inBuffer;
static HIDP_CAPS capabilities;

HANDLE hidHandle;

char msgText[128];      // for logging function
FILE *logStream = NULL; // for logging function

// locally used function prototypes
DWORD WINAPI Read_Thread( LPVOID lpParam );
BOOL bOpenHidDevice(HANDLE *HidDevHandle, USHORT vid, USHORT pid);
int ReadFromConverter(void);

// Function Specification *****************************************************
//
// Name: PrintToLog
//
// Purpose: Opens a log file (if not already open) and prints text to that file
//
// Parms:   outText - zero terminated string to add to the log file
//          
// Returns: 
//
// End Function Specification *************************************************

void PrintToLog(char *outText)
{
   static bool first = true;

   if (first) {
      first = false;
      char logLocation[256];
      strcpy(logLocation, getenv("TEMP"));
      strcat(logLocation, "\\DMXConverterErrorLog.txt");
      logStream = fopen( logLocation, "w" );
   }

   if (logStream) {
      fprintf(logStream, "%s\n", outText);
   }
}

// Function Specification *****************************************************
//
// Name: CloseLog
//
// Purpose: Closes the log file (if it was open).
//
// Parms:
//          
// Returns: 
//
// End Function Specification *************************************************

void CloseLog(void)
{
   if (logStream) {
      fclose(logStream);
   }
}

/**********************************************************
*
*    Function: bOpenHidDevice
*    Purpose: tries to open a HID device based on VID and PID
*    Parameters: vid - HID device's vendor ID
*                pid - HID device's product ID
*                HidDevHandle - pointer to a handle to the HID device
*    Returns: TRUE, if device is found
*             FALSE, if device is not found
*
*   Based on the book "USB Complete, Second Edition" by Jan Axelson
*
**********************************************************/

BOOL bOpenHidDevice(HANDLE *HidDevHandle, USHORT vid, USHORT pid)
{
   static GUID hidGuid;       // HID Globally Unique ID: windows supplies us with this value
   HDEVINFO hidDevInfo;       // handle to structure containing all attached HID Device information
   SP_DEVICE_INTERFACE_DATA devInfoData; // Information structure for HID devices
   BOOL result;               // result of getting next device information structure
   DWORD index;               // index of hidDevInfo array entry
   DWORD dataSize;            // size of the DeviceInterfaceDetail structure
   PSP_DEVICE_INTERFACE_DETAIL_DATA detailData = NULL; // device info data
   HIDD_ATTRIBUTES hidAttrib; // HID device attributes
   UINT lastError;
   UINT bufferSizeError;
   
   // 1) Get the HID Globally Unique ID from the OS
   HidD_GetHidGuid(&hidGuid);
   
   // 2) Get an array of structures containing information about
   // all attached and enumerated HIDs
   hidDevInfo = SetupDiGetClassDevs(&hidGuid, 
      NULL,
      NULL,
      DIGCF_PRESENT | DIGCF_INTERFACEDEVICE);
   
   // 3) Step through the attached device list 1 by 1 and examine
   // each of the attached devices.  When there are no more entries in
   // the array of structures, the function will return FALSE with an error
   // of ERROR_NO_MORE_ITEMS.
   index = 0;         // init to first index of array
   devInfoData.cbSize = sizeof(devInfoData); // set to the size of the structure
                                             // that will contain the device info data
   
   do {
      lastError = NO_ERROR;

      // Get information about the HID device with the 'index' array entry
      result = SetupDiEnumDeviceInterfaces(hidDevInfo, 
         0,
         &hidGuid,
         index,
         &devInfoData);
      
      if (result) {
         // 3) Get the size of the DEVICE_INTERFACE_DETAIL_DATA
         // structure.  This first call will return an error condition, 
         // but we'll get the size of the strucure
         result = SetupDiGetDeviceInterfaceDetail(hidDevInfo,
            &devInfoData,
            NULL,
            0,
            &dataSize,
            NULL);

         if (result == 0) { // error was expected; see if it was the right one
            bufferSizeError = GetLastError();
            if (bufferSizeError == ERROR_INSUFFICIENT_BUFFER) {
               // allocate memory for the hidDevInfo structure
               detailData = (PSP_DEVICE_INTERFACE_DETAIL_DATA) malloc(dataSize);
               
               // set the size parameter of the structure
               detailData->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);
               
               // 4) Now call the function with the correct size parameter.  This 
               // function will return data from one of the array members that 
               // Step #2 pointed to.  This way we can start to identify the
               // attributes of particular HID devices.
               result = SetupDiGetDeviceInterfaceDetail(hidDevInfo,
                  &devInfoData,
                  detailData,
                  dataSize,
                  NULL,
                  NULL);

               if (result) {
                  // 5) Open a file handle to the device.
                  *HidDevHandle = CreateFile( detailData->DevicePath,
                     GENERIC_READ | GENERIC_WRITE,
                     FILE_SHARE_READ | FILE_SHARE_WRITE,
                     NULL,
                     OPEN_EXISTING,
                     0,
                     NULL);

                  // 6) Get the Device VID & PID to see if it's the device we want
                  if(*HidDevHandle != INVALID_HANDLE_VALUE)
                  {
                     hidAttrib.Size = sizeof(hidAttrib);
                     HidD_GetAttributes( *HidDevHandle, &hidAttrib);

                     sprintf(msgText, "Index=%d, VendorID=$%x, ProductID=$%x  "
                             "Device successfully opened for ID checking", 
                             index, hidAttrib.VendorID, hidAttrib.ProductID);
                     PrintToLog(msgText);
                     
                     if((hidAttrib.VendorID == VID) && (hidAttrib.ProductID == PID))
                     {                  
                        // free HID device info list resources
                        SetupDiDestroyDeviceInfoList(hidDevInfo);

                        sprintf(msgText, "Index=%d, Valid Device found", index);
                        PrintToLog(msgText);
                        CloseLog();
                        
                        return TRUE; // found HID device; exit leaving the device open

                     } else {
                        // This Device is not the one we are looking for
                        sprintf(msgText, "Index=%d, Not the device we want", index);
                        PrintToLog(msgText);
                     }
                     
                     // 7) Close the Device Handle because we didn't find the device
                     // with the correct VID and PID
                     CloseHandle(*HidDevHandle);

                  } else {
                     // There was a problem opening the Device
                     lastError = GetLastError();
                     sprintf(msgText, "Index=%d, Error=%d Problem opening the device", index, lastError);
                     PrintToLog(msgText);
                  }
               } else{
                  // There was a problem retrieving the Device Interface Detail
                  lastError = GetLastError();
                  sprintf(msgText, "Index=%d, Error=%d, Problem retrieving the Device Interface Details", 
                          index, lastError);
                  PrintToLog(msgText);
               }
            } else {
               // There was a problem retrieving the required buffer size for the Device
               // Interface Detail; wrong expected error
               lastError = GetLastError();
               sprintf(msgText, "Index=%d, Error=%d, Problem retrieving the required buffer size for "
                       "Device Interface Details", index, lastError);
               PrintToLog(msgText);
            }
         } else {
            // There was a problem retrieving the required buffer size for the Device
            // Interface Detail
            //               there is no lastError
            sprintf(msgText, "Index=%d, Problem retrieving the required buffer size for "
                    "Device Interface Details (no lastError)", index);
            PrintToLog(msgText);
         }

         free(detailData);

      } else {
         // There was a problem retrieving Device Info Data for this index
         lastError = GetLastError();
         if (lastError == ERROR_NO_MORE_ITEMS) {
            sprintf(msgText, "No more devices in list");
         } else {
            sprintf(msgText, "Index=%d, Error=%d, Problem retrieving Device Info Data", index, lastError);
         }
         PrintToLog(msgText);
      }
      
      index++; // increment the array index to search the next entry
      
   } while(lastError != ERROR_NO_MORE_ITEMS); // keep looping until no more items

   PrintToLog("Valid Device was not found");
      
   // free HID device info list resources
   SetupDiDestroyDeviceInfoList(hidDevInfo);

   CloseLog();
   
   return FALSE;
}


void OpenDmxConverter(void)
{
    PHIDP_PREPARSED_DATA hidParsedData;

    // attempt to open the DMX converter (identified by its IDs)
    if (bOpenHidDevice(&hidHandle, VID, PID))
    {
        DMX_adapter_found = true;

        // get a handle to a buffer that describes the devices capabilities
        HidD_GetPreparsedData(hidHandle, &hidParsedData);

        // extract the capabilities info
        HidP_GetCaps(hidParsedData, &capabilities);
            // report sizes are found in
            // capabilities.InputReportByteLength and capabilities.OutputReportByteLength

        outputReportLength = capabilities.OutputReportByteLength;

        // free the memory allocated when getting the preparsed data
        HidD_FreePreparsedData(hidParsedData);

        // create some buffers
        outbuffer = (char *)malloc(capabilities.OutputReportByteLength + 1);
        inBuffer = (char *)malloc(capabilities.InputReportByteLength + 1);

        // request info from the DMX Converter
        SendCommand("I");
    }
}


int ReadFromConverter(void)
{
    BOOL readStat;
    DWORD numCharsRead;

    do
    {
        readStat = ReadFile(hidHandle,
                            inBuffer,
                            capabilities.InputReportByteLength,
                            &numCharsRead,
                            NULL);
        if (readStat)
        {
            // check for "end of file"
            if (numCharsRead <= 0)
            {
                // must be "end of file"
                return -1;
            }
            else
            {
                // NOTE: inBuffer[0] is the USB packet ID
                //       inBuffer[1] contains the transmission flags
                //       inBuffer[2] is the length of data to follow
                //       inBuffer[3..64] is the data

                // terminate the string
                inBuffer[inBuffer[2]+3] = '\0';
            }
        }
        else
        {
            // the read had an error
            MessageBox(NULL, "USB Comm error", "Error", MB_OK);
            // report the error to the caller
            return 1;
        }

      // check for prompt (keep reading until the prompt is found)
    } while (strstr(&inBuffer[3], "\r\n>") == NULL);
    
    // tell the caller it was a good read
    return 0;
}


bool SendCommand(char * command)
{
    BOOL result;
    DWORD numBytesWritten;

    if (DMX_adapter_found)
    {
        // clear the output buffer
        memset(outbuffer, 0, outputReportLength);

        outbuffer[0] = 0;        // USB report ID
        outbuffer[1] = 0x30;    // DMXC flags (RTS=true,DTR=true)

        outbuffer[2] = strlen(command) + 1;            // DMXC command length (data + return)
        sprintf((char *)&outbuffer[3], "%s\r", command);    // the command and return

        result = WriteFile(hidHandle,
                           &outbuffer[0],
                           outputReportLength,
                           &numBytesWritten,
                           NULL);
        if (!result)
        {
            return false;    // return an error indication
        }

        // read the response from the DMX converter
        ReadFromConverter();
    }

    return true;
}
