// DimmerControl.cpp
//

/* Copyright Notice **********************************************************
 * 
 *  Copyright (c) 2004. EdgeView Software, LLC.
 *  All Rights Reserved.
 *  
 *  Permission to use, copy, modify, and distribute this software and its
 *  documentation for any purpose, without fee, and without a written
 *  agreement, is hereby granted, provided that the above copyright
 *  notice, this paragraph and the following three paragraphs appear in
 *  all copies, modifications, and distributions.
 * 
 *  Created by EdgeView Software, LLC. 
 *  For technical information, contact support@edgeviewsoftware.com.
 *  
 *  IN NO EVENT SHALL EDGEVIEW SOFTWARE BE LIABLE TO ANY PARTY FOR DIRECT, 
 *  INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST
 *  PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, 
 *  EVEN IF EDGEVIEW SOFTWARE HAS BEEN ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGES.
 * 
 *  THIS SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF ANY, IS PROVIDED "AS
 *  IS".  EDGEVIEW SOFTWARE HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 *  UPDATES, ENHANCEMENTS, OR MODIFICATIONS TO THIS SOFTWARE.  EDGEVIEW SOFTWARE
 *  SPECIFICALLY DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING,
 *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE. WITHOUT LIMITING THE GENERALITY OF
 *  THE FOREGOING, THIS SOFTWARE IS NOT DESIGNED, MANUFACTURED OR
 *  INTENDED FOR USE OR RESALE IN CONNECTION WITH APPLICATIONS REQUIRING
 *  FAIL SAFE PERFORMANCE (INCLUDING, BUT NOT LIMITED TO, USE IN
 *  HAZARDOUS ENVIRONMENTS, OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 *  NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 *  SUPPORT MACHINES, OR WEAPONS SYSTEMS).
 * 
 * End Copyright Notice ******************************************************/

#include <stdio.h>
#include <tchar.h>
#include "resource.h"
#include "DimmerControl.h"

#define IDC_TRACKBAR 200
#define LABEL_HEIGHT 18
const COLORREF RGB_RED = RGB(255, 0, 0);

#undef  GetWindowID
#define GetWindowID(hwnd) (int)(short)GetDlgCtrlID(hwnd)

#define Trackbar_SetRange (hwnd, redraw, lo, hi) \
                (void)SendMessage((hwnd), TBM_SETRANGE, \
                                  (WPARAM)(BOOL)(redraw), \
                                  MAKELONG((WORD)(lo), (WORD)(hi)))

static DWORD trackID = 0;

DimmerControl::DimmerControl(HWND hWnd,
                 HINSTANCE hInstance)
                 : label1(NULL)
                 , hLabel1(NULL)
{
    m_hWnd = hWnd;
    m_hInstance = hInstance;
}

DimmerControl::~DimmerControl()
{
}

void DimmerControl::SetLabel1(char* label)
{
    if (label1)
    {
        free(label1);
    }
    label1 = _strdup(label);
    if (hLabel1)
    {
        SetWindowText(hLabel1, label1);
    }
}

void DimmerControl::Init(int faderID,
                         int left,
                         int top,
                         int width,
                         int height)
{
    int currentTop;

    controlID = faderID;
    controlLeft = left;
    controlTop = top;
    controlWidth = width;

    if (height == 0)
    { // default height
        // increase default height to accomodate optional features
        sliderHeight = 298;
        currentTop = sliderHeight - 2;
        if (label1)
        {
            label1top = currentTop;
            currentTop += LABEL_HEIGHT;
        }
        frameHeight = currentTop + 2;
    }
    else
    { // specified height
        // fit optional features in the specified height
        currentTop = height - 2;
        if (label1)
        {
            currentTop -= LABEL_HEIGHT;
            label1top = currentTop;
        }
        sliderHeight = currentTop + 2;
        frameHeight = height;
    }

    hDimmerControl = CreateWindow(
        _T("static"),
        NULL,
        WS_CHILD | WS_VISIBLE | SS_BLACKFRAME,
        controlLeft,
        controlTop,
        controlWidth,
        frameHeight,
        m_hWnd,
        (HMENU)faderID,
        m_hInstance,
        NULL);

    hSlider = CreateWindow(
        TRACKBAR_CLASS,
        NULL,
        WS_CHILD | WS_VISIBLE | TBS_VERT | TBS_AUTOTICKS
            | TBS_AUTOTICKS | TBS_LEFT | TBS_BOTH,
        controlLeft+2,
        controlTop+2,
        controlWidth-4,
        sliderHeight-4,
        m_hWnd,
        (HMENU)faderID,
        m_hInstance,
        NULL);
    SendMessage(hSlider, TBM_SETRANGE, (WPARAM)FALSE, MAKELONG(0, 0xFF));
    SendMessage(hSlider, TBM_SETPAGESIZE, 0, (LPARAM)(LONG)0x10);
    SendMessage(hSlider, TBM_SETTICFREQ, (WPARAM)26, (LPARAM)0);
    SetLevel(0);

    if (label1)
    {
        hLabel1 = CreateWindow(
            _T("static"),
            NULL,
            WS_CHILD | WS_VISIBLE | SS_CENTER,
            2,
            label1top,
            controlWidth-4,
            LABEL_HEIGHT,
            hDimmerControl,
            (HMENU)-1,
            m_hInstance,
            NULL);
        SetWindowText(hLabel1, label1);
    }
    else
    {
        hLabel1 = NULL;
    }
}

void DimmerControl::SetLevel(BYTE level)
{
    SendMessage(hSlider, TBM_SETPOS, TRUE, (long)(255 - level));
}

BYTE DimmerControl::GetLevel(void)
{
    return (BYTE)(255 - (int)SendMessage(hSlider, TBM_GETPOS, 0, 0));
}

